import sys
from typing import Any, overload
from typing_extensions import Literal

if sys.platform != "win32" and sys.platform != "darwin":
    AFMT_AC3: int
    AFMT_A_LAW: int
    AFMT_IMA_ADPCM: int
    AFMT_MPEG: int
    AFMT_MU_LAW: int
    AFMT_QUERY: int
    AFMT_S16_BE: int
    AFMT_S16_LE: int
    AFMT_S16_NE: int
    AFMT_S8: int
    AFMT_U16_BE: int
    AFMT_U16_LE: int
    AFMT_U8: int
    SNDCTL_COPR_HALT: int
    SNDCTL_COPR_LOAD: int
    SNDCTL_COPR_RCODE: int
    SNDCTL_COPR_RCVMSG: int
    SNDCTL_COPR_RDATA: int
    SNDCTL_COPR_RESET: int
    SNDCTL_COPR_RUN: int
    SNDCTL_COPR_SENDMSG: int
    SNDCTL_COPR_WCODE: int
    SNDCTL_COPR_WDATA: int
    SNDCTL_DSP_BIND_CHANNEL: int
    SNDCTL_DSP_CHANNELS: int
    SNDCTL_DSP_GETBLKSIZE: int
    SNDCTL_DSP_GETCAPS: int
    SNDCTL_DSP_GETCHANNELMASK: int
    SNDCTL_DSP_GETFMTS: int
    SNDCTL_DSP_GETIPTR: int
    SNDCTL_DSP_GETISPACE: int
    SNDCTL_DSP_GETODELAY: int
    SNDCTL_DSP_GETOPTR: int
    SNDCTL_DSP_GETOSPACE: int
    SNDCTL_DSP_GETSPDIF: int
    SNDCTL_DSP_GETTRIGGER: int
    SNDCTL_DSP_MAPINBUF: int
    SNDCTL_DSP_MAPOUTBUF: int
    SNDCTL_DSP_NONBLOCK: int
    SNDCTL_DSP_POST: int
    SNDCTL_DSP_PROFILE: int
    SNDCTL_DSP_RESET: int
    SNDCTL_DSP_SAMPLESIZE: int
    SNDCTL_DSP_SETDUPLEX: int
    SNDCTL_DSP_SETFMT: int
    SNDCTL_DSP_SETFRAGMENT: int
    SNDCTL_DSP_SETSPDIF: int
    SNDCTL_DSP_SETSYNCRO: int
    SNDCTL_DSP_SETTRIGGER: int
    SNDCTL_DSP_SPEED: int
    SNDCTL_DSP_STEREO: int
    SNDCTL_DSP_SUBDIVIDE: int
    SNDCTL_DSP_SYNC: int
    SNDCTL_FM_4OP_ENABLE: int
    SNDCTL_FM_LOAD_INSTR: int
    SNDCTL_MIDI_INFO: int
    SNDCTL_MIDI_MPUCMD: int
    SNDCTL_MIDI_MPUMODE: int
    SNDCTL_MIDI_PRETIME: int
    SNDCTL_SEQ_CTRLRATE: int
    SNDCTL_SEQ_GETINCOUNT: int
    SNDCTL_SEQ_GETOUTCOUNT: int
    SNDCTL_SEQ_GETTIME: int
    SNDCTL_SEQ_NRMIDIS: int
    SNDCTL_SEQ_NRSYNTHS: int
    SNDCTL_SEQ_OUTOFBAND: int
    SNDCTL_SEQ_PANIC: int
    SNDCTL_SEQ_PERCMODE: int
    SNDCTL_SEQ_RESET: int
    SNDCTL_SEQ_RESETSAMPLES: int
    SNDCTL_SEQ_SYNC: int
    SNDCTL_SEQ_TESTMIDI: int
    SNDCTL_SEQ_THRESHOLD: int
    SNDCTL_SYNTH_CONTROL: int
    SNDCTL_SYNTH_ID: int
    SNDCTL_SYNTH_INFO: int
    SNDCTL_SYNTH_MEMAVL: int
    SNDCTL_SYNTH_REMOVESAMPLE: int
    SNDCTL_TMR_CONTINUE: int
    SNDCTL_TMR_METRONOME: int
    SNDCTL_TMR_SELECT: int
    SNDCTL_TMR_SOURCE: int
    SNDCTL_TMR_START: int
    SNDCTL_TMR_STOP: int
    SNDCTL_TMR_TEMPO: int
    SNDCTL_TMR_TIMEBASE: int
    SOUND_MIXER_ALTPCM: int
    SOUND_MIXER_BASS: int
    SOUND_MIXER_CD: int
    SOUND_MIXER_DIGITAL1: int
    SOUND_MIXER_DIGITAL2: int
    SOUND_MIXER_DIGITAL3: int
    SOUND_MIXER_IGAIN: int
    SOUND_MIXER_IMIX: int
    SOUND_MIXER_LINE: int
    SOUND_MIXER_LINE1: int
    SOUND_MIXER_LINE2: int
    SOUND_MIXER_LINE3: int
    SOUND_MIXER_MIC: int
    SOUND_MIXER_MONITOR: int
    SOUND_MIXER_NRDEVICES: int
    SOUND_MIXER_OGAIN: int
    SOUND_MIXER_PCM: int
    SOUND_MIXER_PHONEIN: int
    SOUND_MIXER_PHONEOUT: int
    SOUND_MIXER_RADIO: int
    SOUND_MIXER_RECLEV: int
    SOUND_MIXER_SPEAKER: int
    SOUND_MIXER_SYNTH: int
    SOUND_MIXER_TREBLE: int
    SOUND_MIXER_VIDEO: int
    SOUND_MIXER_VOLUME: int

    control_labels: list[str]
    control_names: list[str]

    # TODO: oss_audio_device return type
    @overload
    def open(mode: Literal["r", "w", "rw"]) -> Any: ...
    @overload
    def open(device: str, mode: Literal["r", "w", "rw"]) -> Any: ...

    # TODO: oss_mixer_device return type
    def openmixer(device: str = ...) -> Any: ...

    class OSSAudioError(Exception): ...
    error = OSSAudioError



import logging.config
from collections.abc import Callable
from logging import Logger, LogRecord
from typing import Any

from django.core.management.color import Style
from django.http import HttpRequest, HttpResponse

request_logger: Logger
DEFAULT_LOGGING: Any

def configure_logging(logging_config: str, logging_settings: dict[str, Any]) -> None: ...

class AdminEmailHandler(logging.Handler):
    include_html: bool
    email_backend: str | None
    def __init__(
        self,
        include_html: bool = ...,
        email_backend: str | None = ...,
        reporter_class: str | None = ...,
    ) -> None: ...
    def send_mail(self, subject: str, message: str, *args: Any, **kwargs: Any) -> None: ...
    def connection(self) -> Any: ...
    def format_subject(self, subject: str) -> str: ...

class CallbackFilter(logging.Filter):
    callback: Callable
    def __init__(self, callback: Callable) -> None: ...
    def filter(self, record: str | LogRecord) -> bool: ...

class RequireDebugFalse(logging.Filter):
    def filter(self, record: str | LogRecord) -> bool: ...

class RequireDebugTrue(logging.Filter):
    def filter(self, record: str | LogRecord) -> bool: ...

class ServerFormatter(logging.Formatter):
    default_time_format: str
    style: Style
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def uses_server_time(self) -> bool: ...

def log_response(
    message: str,
    *args: Any,
    response: HttpResponse | None = ...,
    request: HttpRequest | None = ...,
    logger: Logger = ...,
    level: str | None = ...,
    exception: BaseException | None = ...
) -> None: ...



from argparse import ArgumentParser
from collections.abc import Sequence
from typing import Any

from django.core.management.base import BaseCommand

class TemplateCommand(BaseCommand):
    url_schemes: Sequence[str]
    rewrite_template_suffixes: Sequence[tuple[str, str]]
    app_or_project: str
    a_or_an: str
    paths_to_remove: Sequence[Any]
    verbosity: Any
    def add_arguments(self, parser: ArgumentParser) -> None: ...
    def handle(self, app_or_project: str, name: str, target: str | None = ..., **options: Any) -> None: ...
    def handle_template(self, template: str | None, subdir: str | None) -> str: ...
    def validate_name(self, name: str, name_or_dir: str = ...) -> None: ...
    def download(self, url: str) -> str: ...
    def splitext(self, the_path: str) -> tuple[str, str]: ...
    def extract(self, filename: str) -> str: ...
    def is_url(self, template: str) -> bool: ...
    def make_writeable(self, filename: str) -> None: ...



from typing import Any

from ..cmd import Command

PYTHON_SOURCE_EXTENSION: str

class install_lib(Command):
    description: str
    user_options: Any
    boolean_options: Any
    negative_opt: Any
    install_dir: Any
    build_dir: Any
    force: int
    compile: Any
    optimize: Any
    skip_build: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def build(self) -> None: ...
    def install(self): ...
    def byte_compile(self, files) -> None: ...
    def get_outputs(self): ...
    def get_inputs(self): ...



from typing import List, Optional, Union

from nwon_baseline.typings import AnyDict

from nwon_django_toolbox.typings.pydantic_base_django import PydanticBaseDjango


class ErrorResponse(PydanticBaseDjango):
    status_code: int
    message: str
    details: Optional[Union[List[str], List[AnyDict], AnyDict]]
    api_docs: Optional[str]



from django.db.migrations.migration import Migration
from django.db.utils import DatabaseError

class AmbiguityError(Exception): ...
class BadMigrationError(Exception): ...
class CircularDependencyError(Exception): ...
class InconsistentMigrationHistory(Exception): ...
class InvalidBasesError(ValueError): ...
class IrreversibleError(RuntimeError): ...

class NodeNotFoundError(LookupError):
    message: str
    origin: Migration | None
    node: tuple[str, str]
    def __init__(self, message: str, node: tuple[str, str], origin: Migration | None = ...) -> None: ...

class MigrationSchemaMissing(DatabaseError): ...
class InvalidMigrationPlan(ValueError): ...



from collections.abc import Sequence
from typing import Any, Generic, TypeVar

from django.core.paginator import Page, Paginator, _SupportsPagination
from django.db.models import Model
from django.http import HttpRequest, HttpResponse
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View

_M = TypeVar("_M", bound=Model, covariant=True)

class MultipleObjectMixin(Generic[_M], ContextMixin):
    allow_empty: bool
    queryset: _SupportsPagination[_M] | None
    model: type[_M] | None
    paginate_by: int
    paginate_orphans: int
    context_object_name: str | None
    paginator_class: type[Paginator]
    page_kwarg: str
    ordering: str | Sequence[str] | None
    def get_queryset(self) -> _SupportsPagination[_M]: ...
    def get_ordering(self) -> str | Sequence[str] | None: ...
    def paginate_queryset(
        self, queryset: _SupportsPagination[_M], page_size: int
    ) -> tuple[Paginator, Page, _SupportsPagination[_M], bool]: ...
    def get_paginate_by(self, queryset: _SupportsPagination[_M]) -> int | None: ...
    def get_paginator(
        self,
        queryset: _SupportsPagination[_M],
        per_page: int,
        orphans: int = ...,
        allow_empty_first_page: bool = ...,
        **kwargs: Any
    ) -> Paginator: ...
    def get_paginate_orphans(self) -> int: ...
    def get_allow_empty(self) -> bool: ...
    def get_context_object_name(self, object_list: _SupportsPagination[_M]) -> str | None: ...

class BaseListView(MultipleObjectMixin[_M], View):
    object_list: _SupportsPagination[_M]
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...

class MultipleObjectTemplateResponseMixin(TemplateResponseMixin):
    template_name_suffix: str

class ListView(MultipleObjectTemplateResponseMixin, BaseListView[_M]): ...



from collections.abc import Iterator
from typing import Any

class Tok:
    num: int
    id: int
    name: str
    regex: str
    next: str | None
    def __init__(self, name: str, regex: str, next: str | None = ...) -> None: ...

def literals(choices: str, prefix: str = ..., suffix: str = ...) -> str: ...

class Lexer:
    regexes: Any
    toks: dict[str, Tok]
    state: str
    def __init__(self, states: dict[str, list[Tok]], first: str) -> None: ...
    def lex(self, text: str) -> Iterator[tuple[str, str]]: ...

class JsLexer(Lexer):
    both_before: list[Tok]
    both_after: list[Tok]
    states: dict[str, list[Tok]]
    def __init__(self) -> None: ...

def prepare_js_for_gettext(js: str) -> str: ...



from _typeshed import AnyPath
from typing import Dict, List, Optional, Tuple

class NetrcParseError(Exception):
    filename: Optional[str]
    lineno: Optional[int]
    msg: str
    def __init__(self, msg: str, filename: Optional[AnyPath] = ..., lineno: Optional[int] = ...) -> None: ...

# (login, account, password) tuple
_NetrcTuple = Tuple[str, Optional[str], Optional[str]]

class netrc:
    hosts: Dict[str, _NetrcTuple]
    macros: Dict[str, List[str]]
    def __init__(self, file: Optional[AnyPath] = ...) -> None: ...
    def authenticators(self, host: str) -> Optional[_NetrcTuple]: ...



import sys
from typing import IO, Any, Dict as DictT, Mapping, MutableMapping, Optional, Text, Type, Union

if sys.version_info >= (3,):
    from enum import Enum
    class PlistFormat(Enum):
        FMT_XML: int
        FMT_BINARY: int
    FMT_XML = PlistFormat.FMT_XML
    FMT_BINARY = PlistFormat.FMT_BINARY

_Path = Union[str, Text]

if sys.version_info >= (3, 9):
    def load(fp: IO[bytes], *, fmt: Optional[PlistFormat] = ..., dict_type: Type[MutableMapping[str, Any]] = ...) -> Any: ...
    def loads(value: bytes, *, fmt: Optional[PlistFormat] = ..., dict_type: Type[MutableMapping[str, Any]] = ...) -> Any: ...

elif sys.version_info >= (3, 4):
    def load(
        fp: IO[bytes],
        *,
        fmt: Optional[PlistFormat] = ...,
        use_builtin_types: bool = ...,
        dict_type: Type[MutableMapping[str, Any]] = ...,
    ) -> Any: ...
    def loads(
        value: bytes,
        *,
        fmt: Optional[PlistFormat] = ...,
        use_builtin_types: bool = ...,
        dict_type: Type[MutableMapping[str, Any]] = ...,
    ) -> Any: ...

if sys.version_info >= (3, 4):
    def dump(
        value: Mapping[str, Any], fp: IO[bytes], *, fmt: PlistFormat = ..., sort_keys: bool = ..., skipkeys: bool = ...
    ) -> None: ...
    def dumps(value: Mapping[str, Any], *, fmt: PlistFormat = ..., skipkeys: bool = ..., sort_keys: bool = ...) -> bytes: ...

if sys.version_info < (3, 9):
    def readPlist(pathOrFile: Union[_Path, IO[bytes]]) -> Any: ...
    def writePlist(value: Mapping[str, Any], pathOrFile: Union[_Path, IO[bytes]]) -> None: ...
    def readPlistFromBytes(data: bytes) -> Any: ...
    def writePlistToBytes(value: Mapping[str, Any]) -> bytes: ...

if sys.version_info < (3,):
    def readPlistFromResource(path: _Path, restype: str = ..., resid: int = ...) -> Any: ...
    def writePlistToResource(rootObject: Mapping[str, Any], path: _Path, restype: str = ..., resid: int = ...) -> None: ...
    def readPlistFromString(data: str) -> Any: ...
    def writePlistToString(rootObject: Mapping[str, Any]) -> str: ...

if sys.version_info < (3, 7):
    class Dict(DictT[str, Any]):
        def __getattr__(self, attr: str) -> Any: ...
        def __setattr__(self, attr: str, value: Any) -> None: ...
        def __delattr__(self, attr: str) -> None: ...

if sys.version_info < (3, 9):
    class Data:
        data: bytes
        def __init__(self, data: bytes) -> None: ...

if sys.version_info >= (3, 8):
    class UID:
        data: int
        def __init__(self, data: int) -> None: ...
        def __index__(self) -> int: ...
        def __reduce__(self) -> Any: ...
        def __hash__(self) -> int: ...

class InvalidFileException(ValueError):
    def __init__(self, message: str = ...) -> None: ...



import sys

if sys.version_info < (3, 8):
    import os

    DEV_NULL = os.devnull
from typing import NamedTuple

if sys.version_info >= (3, 8):
    def libc_ver(executable: str | None = None, lib: str = "", version: str = "", chunksize: int = 16384) -> tuple[str, str]: ...

else:
    def libc_ver(
        executable: str = sys.executable, lib: str = "", version: str = "", chunksize: int = 16384
    ) -> tuple[str, str]: ...

if sys.version_info < (3, 8):
    def linux_distribution(
        distname: str = "",
        version: str = "",
        id: str = "",
        supported_dists: tuple[str, ...] = ...,
        full_distribution_name: bool = ...,
    ) -> tuple[str, str, str]: ...
    def dist(
        distname: str = "", version: str = "", id: str = "", supported_dists: tuple[str, ...] = ...
    ) -> tuple[str, str, str]: ...

def win32_ver(release: str = "", version: str = "", csd: str = "", ptype: str = "") -> tuple[str, str, str, str]: ...

if sys.version_info >= (3, 8):
    def win32_edition() -> str: ...
    def win32_is_iot() -> bool: ...

def mac_ver(
    release: str = "", versioninfo: tuple[str, str, str] = ("", "", ""), machine: str = ""
) -> tuple[str, tuple[str, str, str], str]: ...
def java_ver(
    release: str = "", vendor: str = "", vminfo: tuple[str, str, str] = ("", "", ""), osinfo: tuple[str, str, str] = ("", "", "")
) -> tuple[str, str, tuple[str, str, str], tuple[str, str, str]]: ...
def system_alias(system: str, release: str, version: str) -> tuple[str, str, str]: ...
def architecture(executable: str = sys.executable, bits: str = "", linkage: str = "") -> tuple[str, str]: ...

class uname_result(NamedTuple):
    system: str
    node: str
    release: str
    version: str
    machine: str
    processor: str

def uname() -> uname_result: ...
def system() -> str: ...
def node() -> str: ...
def release() -> str: ...
def version() -> str: ...
def machine() -> str: ...
def processor() -> str: ...
def python_implementation() -> str: ...
def python_version() -> str: ...
def python_version_tuple() -> tuple[str, str, str]: ...
def python_branch() -> str: ...
def python_revision() -> str: ...
def python_build() -> tuple[str, str]: ...
def python_compiler() -> str: ...
def platform(aliased: bool = ..., terse: bool = ...) -> str: ...

if sys.version_info >= (3, 10):
    def freedesktop_os_release() -> dict[str, str]: ...



from typing import Any

from django.utils.connection import BaseConnectionHandler

from .backends.base import BaseCache as BaseCache
from .backends.base import CacheKeyWarning as CacheKeyWarning
from .backends.base import InvalidCacheBackendError as InvalidCacheBackendError

DEFAULT_CACHE_ALIAS: str

class CacheHandler(BaseConnectionHandler):
    settings_name: str
    exception_class: type[Exception]
    def create_connection(self, alias: str) -> BaseCache: ...
    def all(self, initialized_only: bool = ...) -> list[BaseCache]: ...

def close_caches(**kwargs: Any) -> None: ...

cache: BaseCache
caches: CacheHandler



from _typeshed import Incomplete
from typing import Any, NamedTuple
from typing_extensions import Final, Literal, TypeAlias

_TagType: TypeAlias = Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 16]
_TagTuple: TypeAlias = tuple[str, _TagType, int] | tuple[str, _TagInfo, int, dict[str, int]]

class _TagInfo(NamedTuple):
    value: Any
    name: str
    type: _TagType
    length: int
    enum: dict[str, int]

class TagInfo(_TagInfo):
    def __new__(
        cls,
        value: Incomplete | None = None,
        name: str = "unknown",
        type: _TagType | None = None,
        length: int | None = None,
        enum: dict[str, int] | None = None,
    ): ...
    def cvt_enum(self, value): ...

def lookup(tag: int, group: int | None = None) -> _TagInfo: ...

BYTE: Final = 1
ASCII: Final = 2
SHORT: Final = 3
LONG: Final = 4
RATIONAL: Final = 5
SIGNED_BYTE: Final = 6
UNDEFINED: Final = 7
SIGNED_SHORT: Final = 8
SIGNED_LONG: Final = 9
SIGNED_RATIONAL: Final = 10
FLOAT: Final = 11
DOUBLE: Final = 12
IFD: Final = 13
LONG8: Final = 16
TAGS_V2: dict[int, _TagTuple]
TAGS_V2_GROUPS: dict[int, dict[int, _TagTuple]]
TAGS: dict[int, str]
TYPES: dict[int, str]
LIBTIFF_CORE: set[int]



import sys
from collections.abc import Sequence
from typing_extensions import Final

if sys.version_info >= (3, 9):
    __all__ = ["iskeyword", "issoftkeyword", "kwlist", "softkwlist"]
else:
    __all__ = ["iskeyword", "kwlist"]

def iskeyword(s: str) -> bool: ...

# a list at runtime, but you're not meant to mutate it;
# type it as a sequence
kwlist: Final[Sequence[str]]

if sys.version_info >= (3, 9):
    def issoftkeyword(s: str) -> bool: ...

    # a list at runtime, but you're not meant to mutate it;
    # type it as a sequence
    softkwlist: Final[Sequence[str]]



from django.db.backends.base.creation import BaseDatabaseCreation
from django.db.backends.postgresql.base import DatabaseWrapper

class DatabaseCreation(BaseDatabaseCreation):
    connection: DatabaseWrapper



from collections.abc import Iterable
from decimal import Decimal

def format(
    number: Decimal | float | str,
    decimal_sep: str,
    decimal_pos: int | None = ...,
    grouping: int | Iterable[int] = ...,
    thousand_sep: str = ...,
    force_grouping: bool = ...,
    use_l10n: bool | None = ...,
) -> str: ...



from django.urls import _AnyURL

def i18n_patterns(*urls: _AnyURL, prefix_default_language: bool = ...) -> list[_AnyURL]: ...
def is_language_prefix_patterns_used(urlconf: str) -> tuple[bool, bool]: ...

urlpatterns: list[_AnyURL]



from typing import Optional

from amqp import ConnectionError

class KombuError(Exception): ...
class OperationalError(KombuError): ...
class SerializationError(KombuError): ...
class EncodeError(SerializationError): ...
class DecodeError(SerializationError): ...
class NotBoundError(KombuError): ...
class MessageStateError(KombuError): ...
class LimitExceeded(KombuError): ...
class ConnectionLimitExceeded(LimitExceeded): ...
class ChannelLimitExceeded(LimitExceeded): ...
class VersionMismatch(KombuError): ...
class SerializerNotInstalled(KombuError): ...
class ContentDisallowed(SerializerNotInstalled): ...
class InconsistencyError(ConnectionError): ...

class HttpError(Exception):
    def __init__(
        self, code: int, message: Optional[str] = ..., response: Optional[object] = ...
    ) -> None: ...



from django.db.models.fields import Field

from .base import Operation

class FieldOperation(Operation):
    model_name: str
    name: str
    def __init__(self, model_name: str, name: str, field: Field | None = ...) -> None: ...
    @property
    def name_lower(self) -> str: ...
    @property
    def model_name_lower(self) -> str: ...
    def is_same_model_operation(self, operation: FieldOperation) -> bool: ...
    def is_same_field_operation(self, operation: FieldOperation) -> bool: ...

class AddField(FieldOperation):
    field: Field
    preserve_default: bool
    def __init__(self, model_name: str, name: str, field: Field, preserve_default: bool = ...) -> None: ...

class RemoveField(FieldOperation): ...

class AlterField(FieldOperation):
    field: Field
    preserve_default: bool
    def __init__(self, model_name: str, name: str, field: Field, preserve_default: bool = ...) -> None: ...

class RenameField(FieldOperation):
    old_name: str
    new_name: str
    def __init__(self, model_name: str, old_name: str, new_name: str) -> None: ...
    @property
    def old_name_lower(self) -> str: ...
    @property
    def new_name_lower(self) -> str: ...



import _typeshed
from typing import Any, NewType, TypeVar

_T = TypeVar("_T")

_CacheToken = NewType("_CacheToken", int)

def get_cache_token() -> _CacheToken: ...

class ABCMeta(type):
    def __new__(
        __mcls: type[_typeshed.Self], __name: str, __bases: tuple[type[Any], ...], __namespace: dict[str, Any]
    ) -> _typeshed.Self: ...
    def register(cls, subclass: type[_T]) -> type[_T]: ...



from collections.abc import Iterable
from typing import Any

from django.core.management.base import BaseCommand
from django.db.backends.base.base import BaseDatabaseWrapper

class Command(BaseCommand):
    stealth_options: tuple[str]
    db_module: str
    def handle_inspection(self, options: dict[str, Any]) -> Iterable[str]: ...
    def normalize_col_name(
        self, col_name: str, used_column_names: list[str], is_relation: bool
    ) -> tuple[str, dict[str, str], list[str]]: ...
    def get_field_type(
        self, connection: BaseDatabaseWrapper, table_name: str, row: Any
    ) -> tuple[str, dict[str, Any], list[str]]: ...
    def get_meta(
        self, table_name: str, constraints: Any, column_to_field_name: Any, is_view: Any, is_partition: Any
    ) -> list[str]: ...



from enum import Enum
from typing import Any, Callable, Optional, Sequence, Union

from kombu.connection import Connection
from kombu.entity import Exchange, Queue
from kombu.message import Message
from kombu.transport.base import Channel

class Producer:
    def __init__(
        self,
        channel: Union[Connection, Channel],
        exchange: Optional[Union[Exchange, str]] = ...,
        routing_key: Optional[str] = ...,
        serializer: Optional[str] = ...,
        auto_declare: Optional[bool] = ...,
        compression: Optional[str] = ...,
        on_return: Optional[
            Callable[[Exception, Union[Exchange, str], str, Message], None]
        ] = ...,
    ) -> None: ...
    def publish(
        self,
        body: Any,
        routing_key: Optional[str] = ...,
        delivery_mode: Optional[Enum] = ...,
        mandatory: bool = ...,
        immediate: bool = ...,
        priority: int = ...,
        content_type: Optional[str] = ...,
        content_encoding: Optional[str] = ...,
        serializer: Optional[str] = ...,
        headers: Optional[dict[Any, Any]] = ...,
        compression: Optional[str] = ...,
        exchange: Optional[Union[Exchange, str]] = ...,
        retry: bool = ...,
        retry_policy: Optional[dict[Any, Any]] = ...,
        declare: Sequence[Union[Exchange, Queue]] = ...,
        expiration: Optional[float] = ...,
        timeout: Optional[float] = ...,
        **properties: Any
    ) -> None: ...

class Consumer:
    def __init__(
        self,
        channel: Union[Connection, Channel],
        queues: Optional[Sequence[Queue]] = ...,
        no_ack: Optional[bool] = ...,
        auto_declare: Optional[bool] = ...,
        callbacks: Optional[Sequence[Callable[[Any, Message], None]]] = ...,
        on_decode_error: Optional[Callable[[Message, Exception], None]] = ...,
        on_message: Optional[Callable[[Message], None]] = ...,
        accept: Optional[Sequence[str]] = ...,
        prefetch_count: Optional[int] = ...,
        tag_prefix: Optional[str] = ...,
    ) -> None: ...



from typing import Any

from django.core.cache.backends.base import BaseCache
from django.http.request import HttpRequest
from django.http.response import HttpResponse, HttpResponseBase

cc_delim_re: Any

def patch_cache_control(response: HttpResponseBase, **kwargs: Any) -> None: ...
def get_max_age(response: HttpResponse) -> int | None: ...
def set_response_etag(response: HttpResponseBase) -> HttpResponseBase: ...
def get_conditional_response(
    request: HttpRequest,
    etag: str | None = ...,
    last_modified: int | None = ...,
    response: HttpResponse | None = ...,
) -> HttpResponse | None: ...
def patch_response_headers(response: HttpResponseBase, cache_timeout: int | None = ...) -> None: ...
def add_never_cache_headers(response: HttpResponseBase) -> None: ...
def patch_vary_headers(response: HttpResponseBase, newheaders: tuple[str]) -> None: ...
def has_vary_header(response: HttpResponse, header_query: str) -> bool: ...
def get_cache_key(
    request: HttpRequest, key_prefix: str | None = ..., method: str = ..., cache: BaseCache | None = ...
) -> str | None: ...
def learn_cache_key(
    request: HttpRequest,
    response: HttpResponse,
    cache_timeout: float | None = ...,
    key_prefix: str | None = ...,
    cache: BaseCache | None = ...,
) -> str: ...



import sys
from types import TracebackType
from typing import IO, Any, Callable, ContextManager, Iterable, Iterator, Optional, Type, TypeVar, overload
from typing_extensions import Protocol

if sys.version_info >= (3, 5):
    from typing import AsyncContextManager, AsyncIterator

if sys.version_info >= (3, 6):
    AbstractContextManager = ContextManager
if sys.version_info >= (3, 7):
    AbstractAsyncContextManager = AsyncContextManager

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T_io = TypeVar("_T_io", bound=Optional[IO[str]])
_F = TypeVar("_F", bound=Callable[..., Any])

_ExitFunc = Callable[[Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]], bool]
_CM_EF = TypeVar("_CM_EF", ContextManager[Any], _ExitFunc)

if sys.version_info >= (3, 2):
    class _GeneratorContextManager(ContextManager[_T_co]):
        def __call__(self, func: _F) -> _F: ...
    def contextmanager(func: Callable[..., Iterator[_T]]) -> Callable[..., _GeneratorContextManager[_T]]: ...

else:
    class GeneratorContextManager(ContextManager[_T_co]):
        def __call__(self, func: _F) -> _F: ...
    def contextmanager(func: Callable[..., Iterator[_T]]) -> Callable[..., ContextManager[_T]]: ...

if sys.version_info >= (3, 7):
    def asynccontextmanager(func: Callable[..., AsyncIterator[_T]]) -> Callable[..., AsyncContextManager[_T]]: ...

if sys.version_info < (3,):
    def nested(*mgr: ContextManager[Any]) -> ContextManager[Iterable[Any]]: ...

class _SupportsClose(Protocol):
    def close(self) -> None: ...

_SupportsCloseT = TypeVar("_SupportsCloseT", bound=_SupportsClose)

class closing(ContextManager[_SupportsCloseT]):
    def __init__(self, thing: _SupportsCloseT) -> None: ...

if sys.version_info >= (3, 10):
    class _SupportsAclose(Protocol):
        async def aclose(self) -> None: ...
    _SupportsAcloseT = TypeVar("_SupportsAcloseT", bound=_SupportsAclose)
    class aclosing(AsyncContextManager[_SupportsAcloseT]):
        def __init__(self, thing: _SupportsAcloseT) -> None: ...

if sys.version_info >= (3, 4):
    class suppress(ContextManager[None]):
        def __init__(self, *exceptions: Type[BaseException]) -> None: ...
        def __exit__(
            self, exctype: Optional[Type[BaseException]], excinst: Optional[BaseException], exctb: Optional[TracebackType]
        ) -> bool: ...
    class redirect_stdout(ContextManager[_T_io]):
        def __init__(self, new_target: _T_io) -> None: ...

if sys.version_info >= (3, 5):
    class redirect_stderr(ContextManager[_T_io]):
        def __init__(self, new_target: _T_io) -> None: ...

if sys.version_info >= (3,):
    class ContextDecorator:
        def __call__(self, func: _F) -> _F: ...
    _U = TypeVar("_U", bound=ExitStack)
    class ExitStack(ContextManager[ExitStack]):
        def __init__(self) -> None: ...
        def enter_context(self, cm: ContextManager[_T]) -> _T: ...
        def push(self, exit: _CM_EF) -> _CM_EF: ...
        def callback(self, callback: Callable[..., Any], *args: Any, **kwds: Any) -> Callable[..., Any]: ...
        def pop_all(self: _U) -> _U: ...
        def close(self) -> None: ...
        def __enter__(self: _U) -> _U: ...
        def __exit__(
            self,
            __exc_type: Optional[Type[BaseException]],
            __exc_value: Optional[BaseException],
            __traceback: Optional[TracebackType],
        ) -> bool: ...

if sys.version_info >= (3, 7):
    from typing import Awaitable

    _S = TypeVar("_S", bound=AsyncExitStack)

    _ExitCoroFunc = Callable[[Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]], Awaitable[bool]]
    _CallbackCoroFunc = Callable[..., Awaitable[Any]]
    _ACM_EF = TypeVar("_ACM_EF", AsyncContextManager[Any], _ExitCoroFunc)
    class AsyncExitStack(AsyncContextManager[AsyncExitStack]):
        def __init__(self) -> None: ...
        def enter_context(self, cm: ContextManager[_T]) -> _T: ...
        def enter_async_context(self, cm: AsyncContextManager[_T]) -> Awaitable[_T]: ...
        def push(self, exit: _CM_EF) -> _CM_EF: ...
        def push_async_exit(self, exit: _ACM_EF) -> _ACM_EF: ...
        def callback(self, callback: Callable[..., Any], *args: Any, **kwds: Any) -> Callable[..., Any]: ...
        def push_async_callback(self, callback: _CallbackCoroFunc, *args: Any, **kwds: Any) -> _CallbackCoroFunc: ...
        def pop_all(self: _S) -> _S: ...
        def aclose(self) -> Awaitable[None]: ...
        def __aenter__(self: _S) -> Awaitable[_S]: ...
        def __aexit__(
            self,
            __exc_type: Optional[Type[BaseException]],
            __exc_value: Optional[BaseException],
            __traceback: Optional[TracebackType],
        ) -> Awaitable[bool]: ...

if sys.version_info >= (3, 7):
    @overload
    def nullcontext(enter_result: _T) -> ContextManager[_T]: ...
    @overload
    def nullcontext() -> ContextManager[None]: ...



from collections.abc import Sequence
from types import ModuleType
from typing import Any

from django.core.cache.backends.base import BaseCache

class BaseMemcachedCache(BaseCache):
    def __init__(
        self,
        server: str | Sequence[str],
        params: dict[str, Any],
        library: ModuleType,
        value_not_found_exception: type[BaseException],
    ) -> None: ...

class MemcachedCache(BaseMemcachedCache):
    def __init__(self, server: str | Sequence[str], params: dict[str, Any]) -> None: ...

class PyLibMCCache(BaseMemcachedCache):
    def __init__(self, server: str | Sequence[str], params: dict[str, Any]) -> None: ...

class PyMemcacheCache(BaseMemcachedCache):
    def __init__(self, server: str | Sequence[str], params: dict[str, Any]) -> None: ...



from typing import Any

from .Image import ImageTransformHandler

class Transform(ImageTransformHandler):
    data: Any
    def __init__(self, data) -> None: ...
    def getdata(self): ...
    def transform(self, size, image, **options): ...

class AffineTransform(Transform):
    method: Any

class ExtentTransform(Transform):
    method: Any

class QuadTransform(Transform):
    method: Any

class MeshTransform(Transform):
    method: Any



def supports_color() -> bool: ...

class Style:
    # set by make_style
    def ERROR(self, text: str) -> str: ...
    def SUCCESS(self, text: str) -> str: ...
    def WARNING(self, text: str) -> str: ...
    def NOTICE(self, text: str) -> str: ...
    def SQL_FIELD(self, text: str) -> str: ...
    def SQL_COLTYPE(self, text: str) -> str: ...
    def SQL_KEYWORD(self, text: str) -> str: ...
    def SQL_TABLE(self, text: str) -> str: ...
    def HTTP_INFO(self, text: str) -> str: ...
    def HTTP_SUCCESS(self, text: str) -> str: ...
    def HTTP_REDIRECT(self, text: str) -> str: ...
    def HTTP_NOT_MODIFIED(self, text: str) -> str: ...
    def HTTP_BAD_REQUEST(self, text: str) -> str: ...
    def HTTP_NOT_FOUND(self, text: str) -> str: ...
    def HTTP_SERVER_ERROR(self, text: str) -> str: ...
    def MIGRATE_HEADING(self, text: str) -> str: ...
    def MIGRATE_LABEL(self, text: str) -> str: ...
    def ERROR_OUTPUT(self, text: str) -> str: ...

def make_style(config_string: str = ...) -> Style: ...
def no_style() -> Style: ...
def color_style(force_color: bool = ...) -> Style: ...



import sys

if sys.platform != "win32":
    from _curses import _CursesWindow

    version: str

    class _Curses_Panel:  # type is <class '_curses_panel.curses panel'> (note the space in the class name)
        def above(self) -> _Curses_Panel: ...
        def below(self) -> _Curses_Panel: ...
        def bottom(self) -> None: ...
        def hidden(self) -> bool: ...
        def hide(self) -> None: ...
        def move(self, y: int, x: int) -> None: ...
        def replace(self, win: _CursesWindow) -> None: ...
        def set_userptr(self, obj: object) -> None: ...
        def show(self) -> None: ...
        def top(self) -> None: ...
        def userptr(self) -> object: ...
        def window(self) -> _CursesWindow: ...

    def bottom_panel() -> _Curses_Panel: ...
    def new_panel(__win: _CursesWindow) -> _Curses_Panel: ...
    def top_panel() -> _Curses_Panel: ...
    def update_panels() -> _Curses_Panel: ...



from typing import Type

from django.db.models import Model
from rest_framework import status
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import Serializer


def update_and_return_in_view(
    instance: Model,
    request: Request,
    update_serializer: Type[Serializer],
    return_serializer: Type[Serializer],
    partial: bool = False,
) -> Response:
    """
    A helper function that is supposed to be used in an update view.

    It takes an update_serializer that serializes the request.data
    and returns the updated model instance via the return_serializer.
    """

    serializer = update_serializer(
        data=request.data,
        instance=instance,
        partial=partial,
    )

    if serializer.is_valid():
        updated_instance = serializer.save()
    else:
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    return Response(
        data=return_serializer(instance=updated_instance).data,
        status=status.HTTP_200_OK,
    )



from datetime import datetime
from typing import Any, TypeVar

from django.contrib.sessions.backends.base import SessionBase
from django.db import models

_T = TypeVar("_T", bound=AbstractBaseSession)

class BaseSessionManager(models.Manager[_T]):
    def encode(self, session_dict: dict[str, Any]) -> str: ...
    def save(self, session_key: str, session_dict: dict[str, Any], expire_date: datetime) -> _T: ...

class AbstractBaseSession(models.Model):
    expire_date: datetime
    session_data: str
    session_key: str
    objects: Any
    @classmethod
    def get_session_store_class(cls) -> type[SessionBase] | None: ...
    def get_decoded(self) -> dict[str, Any]: ...



import gettext as gettext_module
from collections.abc import Iterator
from gettext import NullTranslations
from re import Pattern

# switch to tuple once https://github.com/python/mypy/issues/11098 is fixed
from typing import Any, Protocol, Tuple, TypeVar  # noqa: Y022

from django.http.request import HttpRequest
from typing_extensions import Literal, TypeAlias

CONTEXT_SEPARATOR: Literal["\x04"]
accept_language_re: Pattern[str]
language_code_re: Pattern[str]
language_code_prefix_re: Pattern[str]

class _PluralCallable(Protocol):
    def __call__(self, __n: int) -> int: ...

def reset_cache(**kwargs: Any) -> None: ...

# switch to tuple once https://github.com/python/mypy/issues/11098 is fixed
_KeyT: TypeAlias = str | Tuple[str, int]

_Z = TypeVar("_Z")

class TranslationCatalog:
    _catalogs: list[dict[_KeyT, str]]
    def __init__(self, trans: gettext_module.NullTranslations | None = ...) -> None: ...
    def __getitem__(self, key: _KeyT) -> str: ...
    def __setitem__(self, key: _KeyT, value: str) -> None: ...
    def __contains__(self, key: _KeyT) -> bool: ...
    def items(self) -> Iterator[tuple[_KeyT, str]]: ...
    def keys(self) -> Iterator[_KeyT]: ...
    def update(self, trans: gettext_module.NullTranslations) -> None: ...
    def get(self, key: _KeyT, default: _Z = ...) -> str | _Z: ...
    def plural(self, msgid: str, num: int) -> str: ...

class DjangoTranslation(gettext_module.GNUTranslations):
    domain: str
    plural: _PluralCallable
    def __init__(self, language: str, domain: str | None = ..., localedirs: list[str] | None = ...) -> None: ...
    def merge(self, other: NullTranslations) -> None: ...
    def language(self) -> str: ...
    def to_language(self) -> str: ...
    def ngettext(self, msgid1: str, msgid2: str, n: int) -> str: ...

def translation(language: str) -> DjangoTranslation: ...
def activate(language: str) -> None: ...
def deactivate() -> None: ...
def deactivate_all() -> None: ...
def get_language() -> str: ...
def get_language_bidi() -> bool: ...
def catalog() -> DjangoTranslation: ...
def gettext(message: str) -> str: ...
def pgettext(context: str, message: str) -> str: ...
def gettext_noop(message: str) -> str: ...
def do_ntranslate(singular: str, plural: str, number: float, translation_function: str) -> str: ...
def ngettext(singular: str, plural: str, number: float) -> str: ...
def npgettext(context: str, singular: str, plural: str, number: int) -> str: ...
def all_locale_paths() -> list[str]: ...
def check_for_language(lang_code: str | None) -> bool: ...
def get_languages() -> dict[str, str]: ...
def get_supported_language_variant(lang_code: str | None, strict: bool = ...) -> str: ...
def get_language_from_path(path: str, strict: bool = ...) -> str | None: ...
def get_language_from_request(request: HttpRequest, check_path: bool = ...) -> str: ...
def parse_accept_lang_header(lang_string: str) -> tuple[tuple[str, float], ...]: ...



from collections.abc import Mapping, Sequence
from typing import Any

from django.contrib.gis.db.models.lookups import GISLookup
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.sql.compiler import _AsSqlType

class SpatialOperator:
    sql_template: Any
    op: Any
    func: Any
    def __init__(self, op: Any | None = ..., func: Any | None = ...) -> None: ...
    @property
    def default_template(self) -> Any: ...
    def as_sql(
        self,
        connection: BaseDatabaseWrapper,
        lookup: GISLookup,
        template_params: Mapping[str, Any],
        sql_params: Sequence[Any],
    ) -> _AsSqlType: ...



from django.template.backends.base import BaseEngine
from django.template.base import Origin

class TemplateDoesNotExist(Exception):
    backend: BaseEngine | None
    tried: list[tuple[Origin, str]]
    chain: list[TemplateDoesNotExist]
    def __init__(
        self,
        msg: Origin | str,
        tried: list[tuple[Origin, str]] | None = ...,
        backend: BaseEngine | None = ...,
        chain: list[TemplateDoesNotExist] | None = ...,
    ) -> None: ...

class TemplateSyntaxError(Exception): ...



from django.core.management.base import BaseCommand
from django.db.backends.base.base import BaseDatabaseWrapper

class Command(BaseCommand):
    verbosity: int
    def show_list(self, connection: BaseDatabaseWrapper, app_names: list[str] | None = ...) -> None: ...
    def show_plan(self, connection: BaseDatabaseWrapper, app_names: list[str] | None = ...) -> None: ...



from collections.abc import Sequence
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.backends.ddl_references import Statement
from django.db.models.base import Model
from django.db.models.expressions import BaseExpression, Combinable, Expression, Func
from django.db.models.query_utils import Q
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType

class Index:
    model: type[Model]
    suffix: str
    max_name_length: int
    fields: Sequence[str]
    fields_orders: Sequence[tuple[str, str]]
    name: str
    db_tablespace: str | None
    opclasses: Sequence[str]
    condition: Q | None
    expressions: Sequence[BaseExpression | Combinable]
    include: Sequence[str]
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...
    @property
    def contains_expressions(self) -> bool: ...
    def create_sql(
        self, model: type[Model], schema_editor: BaseDatabaseSchemaEditor, using: str = ..., **kwargs: Any
    ) -> Statement: ...
    def remove_sql(self, model: type[Model], schema_editor: BaseDatabaseSchemaEditor, **kwargs: Any) -> str: ...
    def deconstruct(self) -> Any: ...
    def clone(self) -> Index: ...
    def set_name_with_model(self, model: type[Model]) -> None: ...

class IndexExpression(Func):
    template: str
    wrapper_classes: Sequence[Expression]
    def set_wrapper_classes(self, connection: Any | None = ...) -> None: ...
    @classmethod
    def register_wrappers(cls, *wrapper_classes: Expression) -> None: ...
    def resolve_expression(
        self,
        query: Any | None = ...,
        allow_joins: bool = ...,
        reuse: set[str] | None = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> IndexExpression: ...
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...
